/*
This macro writes field functions to carry out 2-D tabular interpolation in STAR-CCM+.
In the description below the dependent variable is called "D". The 1st independent variable
is called "A" and the 2nd independent variable is called "B".

User Inputs required: (In order of appearance in the code)

1.) in_var2_values[]:
	A range of values as an array for the second independent field function. This can be 
	provided as either a double array or a single array. Comment the desired line in the 
	user inputs. 
	
2.) tableName_prefix 
3.) tableName_suffix
	These variable defines the prefix/suffix for the table names. 
	The macro requires 1D tables between the 1st independent variable "A" and the dependent variable
	at each value of the 2nd independent variable "B" e.g. Values of A vs D; at each value of B.
        If there is no prefix/suffix, then set that variablet to "";
	
	The table should contain the value of the 2nd independent variable "B".
	
	If a double array for is used for definition of in_var2_values[], then the decimal in the value is replaced
	with a "p" in the table name. The macro excepts table names as B_300p0, B_320p0 etc. 
	
	If an integer array is used for the definition of in_var2_values[] then the table names are simply the values
	of the variable B. The macro excepts table names as B_300, B_320 etc. 

4.) tableFilePath
        This variable defines the file path of the directory where all the tables are located. 
        If the tables are located in the same directory as the sim file, then this variable 
        should be set to simDir i.e. tableFilePath=simDir; .
	
5.)	in_var1
	This variable defines the table header of the 1st independent variable in the 1D table. 

6.)	de_var
	This variable defines the table header of the dependent variable in the 1D table.

7.)	in_var1_FF
	This variable defines the name of the field function in the simulation for the 1st
	independent variable against which we wish to make the interpolation from the table.
	The macro assumes that this field function exists in the simulaiton. 

8.) in_var2_FF
	This variable defines the name of the field function in the simulation for the 2nd
	independent variable against which we wish to make the interpolation from the table.
	The macro assumes that this field function exists in the simulaiton. 
	
9.) in_var1_interp_FFName_prefix
10.)in_var1_interp_FFName_suffix
	Name of the field function created by the macro that contains the 1D interpolation of 
	the 1st independent variable. These field functions will contain the value of the 2nd
        independent variable. Therefore the user can specify a prefix name and suffix name.
        If there is no prefix/suffix, then set that variablet to empty i.e. "";
        

*/
package macro;

import java.util.*;

import star.common.*;
import star.base.neo.*;

public class TwoD_Table_Interpolation extends StarMacro {

  public void execute() {
    execute0();
  }

  private void execute0() {
    
    Simulation sim =getActiveSimulation();
    String simDir = sim.getSessionDir();
      
    //#################################################
    //User Inputs
    //#################################################
    //Define the values for the 2nd independent variable. 
    
    //uncomment the line below depending on if you wish to use double or integer for the 2nd variable.
    //double in_var2_values[]  = {300.0,320.0,340.0}; //The values are assumed to be in ascending order
    int in_var2_values[]  = {300,320,340,360};
   
    /*Define table name prefix.
    Table names are assumed to contain the value
    of the 2nd independent variables e.g. B_300p0, B_320p0 etc. 
    If double array is used then the decimal is replaced by a "p"
    in the table name. For integers table names are B_300, B_320 etc.
    If there is no suffix/prefix, set the variable to ""
    */
    String tableName_prefix = "B_";
    String tableName_suffix = "";
    
    /*
    Define the location of table file path. 
    If table are stored in the same location as the sim file then set the variable
    below to simDir
    */
    String tableFilePath = simDir;
    
    
    //Define name of tabel headers
    String in_var1 = "A"; //1st independent variable
    String de_var = "D"; //dependent variable
    
    //Defining independent variable field functions
    //It is assumed that these field functions are already present in the sim file
    String in_var1_FF = "A";
    String in_var2_FF = "B";
    
    /*
    Define the name of the field function that interpolates w.r.t to the 1st independent variable.
    The field function created for interpolation of
    the 1st independent variable contain the value of
    2nd variable. If there is no suffix/prefix, set the variable to ""
    */
    String in_var1_interp_FFName_prefix = "A_at_B_";
    String in_var1_interp_FFName_suffix = "";
    
    //Name of the final interpolated field funciton
    String final_Interpolated_FFName ="final_Interp_FF";
    
    //#################################################
    //Don't edit below this line
    //#################################################
 
    //Defining additional variables 
    String in_var2_tablename[]=new String[in_var2_values.length]; //Array to store table names
    String in_var1_interp_FF[]=new String[in_var2_values.length]; //Array to store FF names for 1st variable interpolation FF's
    
    //Adjusting FF name to field function syntax
    in_var1_FF = FFsyntax(in_var1_FF);
    in_var2_FF = FFsyntax(in_var2_FF);
    
    //Creating tabular interpolation field functions for 1st variable. 
    for(int i=0;i<in_var2_values.length;i++){
        //Replacing the decimal in the double value by "p" for getting the table names. 
        String tableNameValue = String.valueOf(in_var2_values[i]).replaceAll("\\.", "p");
        in_var2_tablename[i] = tableName_prefix +  String.valueOf(tableNameValue)+tableName_suffix;
        
        //importing tables in STAR-CCM+
        //Tables are assumed to be in the same directory as the sim file. 
        sim.getTableManager().createFromFile(resolvePath(tableFilePath+"/"+in_var2_tablename[i]+".csv"));
         
        //Writing interpolate FF for 1st variable
        UserFieldFunction interpolateFF1 = sim.getFieldFunctionManager().createFieldFunction();
        interpolateFF1.setPresentationName(in_var1_interp_FFName_prefix+ tableNameValue+in_var1_interp_FFName_suffix);
        interpolateFF1.setFunctionName(interpolateFF1.getPresentationName());
        interpolateFF1.getTypeOption().setSelected(FieldFunctionTypeOption.Type.SCALAR);
        interpolateFF1.setDefinition("interpolateTable(@Table(\""+in_var2_tablename[i]+"\"), \""+in_var1+"\", LINEAR, \""+de_var+"\", "+in_var1_FF+")");
        
        //Storing the name of the FF's created above in an array. 
        in_var1_interp_FF[i]=FFsyntax(in_var1_interp_FFName_prefix+ tableNameValue +in_var1_interp_FFName_suffix);
    }
    
    //Defining 2D interpolating field function. 
   
    //Defining field function line if the independent variable is out of range. 
    
    //Defining array to store string value of the FF definition. 
    String [] sec_interp_FF_def = new String[in_var2_values.length + 1]; 
    //Defintion for below min
    sec_interp_FF_def[0] = in_var2_FF+ "<=" +in_var2_values[0]+ " ? " +in_var1_interp_FF[0]+ " :  \n";
    //Definition above max
    sec_interp_FF_def[sec_interp_FF_def.length -1] = in_var2_FF+ ">=" +in_var2_values[in_var2_values.length-1]+ " ? " +in_var1_interp_FF[in_var2_values.length-1]+ " : 0 \n";
    
    //Defining linear interpolation FF definition for intermediate ranges.
    int i=0;
    int j=1;
    while (j<in_var2_values.length) {
        sec_interp_FF_def[j] = linearEqString(String.valueOf(in_var2_values[i]),String.valueOf(in_var2_values[j]),in_var1_interp_FF[i],in_var1_interp_FF[j],in_var2_FF);
        i++;
        j++;
    }
    
    //Building the string defintion from the individual definitions stored in the array. 
    StringBuilder strBuilder = new StringBuilder();
    for (int n = 0; n < sec_interp_FF_def.length; n++) {
        strBuilder.append(sec_interp_FF_def[n]);
    }
    String sec_interp_FF_def_string = strBuilder.toString();
    
    //Creating field function for 2nd variable interpolation
    UserFieldFunction interpolateFF2 = sim.getFieldFunctionManager().createFieldFunction();
    interpolateFF2.setPresentationName(final_Interpolated_FFName);
    interpolateFF2.setFunctionName(interpolateFF2.getPresentationName());
    interpolateFF2.getTypeOption().setSelected(FieldFunctionTypeOption.Type.SCALAR);
    interpolateFF2.setDefinition(sec_interp_FF_def_string);

  }
  
  public String linearEqString (String x1, String x2, String y1, String y2,String X) {
    //This function defines an if statement with a linear interpolation equation.
	//Defining equation of gradient calculation
    String m ="((" +y2+ "-"  +y1+ ")/(" +x2+ "-" +x1+ "))"; 
    //Defining equation of a linear line calculation
    String Eq = m+ "*(" +X+  "-" +x1+ ") + " +y1;
   
    //Defining an if statement for the ranges based on X
    String STAR_if = X+ ">" +x1+ " && " +X+ "<=" +x2+ " ? " +Eq+ "  : \n";
            
    return STAR_if;
  }
  
  
  public String FFsyntax (String FFname){
	//This functions writes the FF name into field function variable. 
    String FFsyntax = "${"+FFname+"}";
    
    return FFsyntax;

  }
  
}
